@@ -1,33 +1,10 @@
-import _patch       # NOQA
+from _patch import zmq, USING_GEVENT  # NOQA
 import logging
 import os
 import sys
 
 
-try:
-    from gevent import monkey       # NOQA
-    try:
-        from gevent_zeromq import monkey_patch, IOLOOP_IS_MONKEYPATCHED  # NOQA
-        monkey.patch_all()
-        monkey_patch()
-    except ImportError:
-        msg = """We have detected that you have gevent in your
-        environment. In order to have Circus working, you *must*
-        install gevent_zmq from :
-
-          https://github.com/tarekziade/gevent-zeromq
-
-        Circus will not need this in the future once
-        pyzmq gets a green poller:
-
-          https://github.com/zeromq/pyzmq/issues/197
-        """
-        raise ImportError(msg)
-except ImportError:
-    pass
-
-
-version_info = (0, 5, 2)
+version_info = (0, 6, 0)
 __version__ = ".".join(map(str, version_info))
 
 
@@ -39,7 +16,7 @@ def get_arbiter(watchers, controller=None,
                 stats_endpoint=None,
                 env=None, name=None, context=None,
                 background=False, stream_backend="thread",
-                plugins=None, debug=False):
+                plugins=None, debug=False, proc_name="circusd"):
     """Creates a Arbiter and a single watcher in it.
 
     Options:
@@ -85,6 +62,13 @@ def get_arbiter(watchers, controller=None,
             - any other key will be passed the class constructor.
         - **max_retry**: the number of times we attempt to start a process,
           before we abandon and stop the whole watcher. (default: 5)
+        - **hooks**: callback functions for hooking into the watcher startup
+          and shutdown process. **hooks** is a dict where each key is the hook
+          name and each value is a 2-tuple with the name of the callable
+          or the callabled itself and a boolean flag indicating if an
+          exception occuring in the hook should not be ignored.
+          Possible values for the hook name: *before_start*, *after_start*,
+          *before_stop*, *after_stop*.
 
     - **controller** -- the zmq entry point (default: 'tcp://127.0.0.1:5555')
     - **pubsub_endpoint** -- the zmq entry point for the pubsub
@@ -104,6 +88,7 @@ def get_arbiter(watchers, controller=None,
 
     - **debug** -- If True the arbiter is launched in debug mode
       (default: False)
+    - **proc_name** -- the arbiter process name (default: circusd)
     """
     from circus.util import DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB
     if controller is None:
@@ -113,19 +98,13 @@ def get_arbiter(watchers, controller=None,
 
     if stream_backend == 'gevent':
         try:
-            import gevent           # NOQA
-            import gevent_zeromq    # NOQA
+            import gevent                   # NOQA
         except ImportError:
             sys.stderr.write("stream_backend set to gevent, " +
-                             "but gevent or gevent_zeromq isn't installed\n")
+                             "but gevent isn't installed\n")
             sys.stderr.write("Exiting...")
             sys.exit(1)
 
-        from gevent import monkey
-        from gevent_zeromq import monkey_patch
-        monkey.patch_all()
-        monkey_patch()
-
     from circus.watcher import Watcher
     if background:
         from circus.arbiter import ThreadedArbiter as Arbiter   # NOQA
@@ -142,4 +121,5 @@ def get_arbiter(watchers, controller=None,
 
     return Arbiter(_watchers, controller, pubsub_endpoint,
                    stats_endpoint=stats_endpoint,
-                   context=context, plugins=plugins, debug=debug)
+                   context=context, plugins=plugins, debug=debug,
+                   proc_name=proc_name)
