@@ -64,9 +64,11 @@ class List(Command):
             return {"watchers": [name for name in watchers]}
 
     def console_msg(self, msg):
-        if "processes" in msg:
+        if "pids" in msg:
             return ",".join([str(process_id)
                              for process_id in msg.get('pids')])
         elif 'watchers' in msg:
             return ",".join([watcher for watcher in msg.get('watchers')])
+        if 'reason' not in msg:
+            msg['reason'] = "Response doesn't contain 'pids' nor 'watchers'."
         return self.console_error(msg)
