@@ -1,10 +1,11 @@
 import os
 import fnmatch
 import sys
+import warnings
 
 from circus import logger
 from circus.util import (DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB,
-                         StrictConfigParser)
+                         StrictConfigParser, parse_env_str)
 
 
 def watcher_defaults():
@@ -29,7 +30,20 @@ def watcher_defaults():
         'use_sockets': False,
         'singleton': False,
         'copy_env': False,
-        'copy_path': False}
+        'copy_path': False,
+        'hooks': dict(),
+        'respawn': True}
+
+
+_BOOL_STATES = {'1': True, 'yes': True, 'true': True, 'on': True,
+                '0': False, 'no': False, 'false': False, 'off': False}
+
+
+def to_boolean(value):
+    value = value.lower().strip()
+    if value not in _BOOL_STATES:
+        raise ValueError(value)
+    return _BOOL_STATES[value]
 
 
 class DefaultConfigParser(StrictConfigParser):
@@ -89,7 +103,7 @@ def read_config(config_path):
 def get_config(config_file):
     if not os.path.exists(config_file):
         sys.stderr.write("the configuration file %r does not exist\n" %
-                config_file)
+                         config_file)
         sys.stderr.write("Exiting...\n")
         sys.exit(1)
 
@@ -112,22 +126,35 @@ def get_config(config_file):
     if stream_backend == 'gevent':
         try:
             import gevent           # NOQA
-            import gevent_zeromq    # NOQA
         except ImportError:
             sys.stderr.write("stream_backend set to gevent, " +
-                             "but gevent or gevent_zeromq isn't installed\n")
-            sys.stderr.write("Exiting...")
+                             "but gevent isn't installed\n")
+            sys.stderr.write("Exiting...\n")
             sys.exit(1)
 
         from gevent import monkey
-        from gevent_zeromq import monkey_patch
         monkey.patch_all()
-        monkey_patch()
+
+        try:
+            import zmq.green as zmq         # NOQA
+        except ImportError:
+            try:
+                from gevent_zeromq import monkey_patch
+            except ImportError:
+                sys.stderr.write("stream_backend set to gevent, but " +
+                                 "but required PyZMQ >= 2.2.0.1 not found\n")
+                sys.stderr.write("Exiting...\n")
+                sys.exit(1)
+
+            monkey_patch()
+            warnings.warn("gevent_zeromq is deprecated, please "
+                          "use PyZMQ >= 2.2.0.1")
 
     config['stream_backend'] = stream_backend
 
     # Initialize watchers, plugins & sockets to manage
     watchers = []
+    environs = {}
     plugins = []
     sockets = []
 
@@ -152,13 +179,13 @@ def get_config(config_file):
                     watcher['args'] = val
                 elif opt == 'numprocesses':
                     watcher['numprocesses'] = dget(section, 'numprocesses', 1,
-                            int)
+                                                   int)
                 elif opt == 'warmup_delay':
                     watcher['warmup_delay'] = dget(section, 'warmup_delay', 0,
-                            int)
+                                                   int)
                 elif opt == 'executable':
                     watcher['executable'] = dget(section, 'executable', None,
-                            str)
+                                                 str)
                 elif opt == 'working_dir':
                     watcher['working_dir'] = val
                 elif opt == 'shell':
@@ -169,15 +196,15 @@ def get_config(config_file):
                     watcher['gid'] = val
                 elif opt == 'send_hup':
                     watcher['send_hup'] = dget(section, 'send_hup', False,
-                            bool)
+                                               bool)
                 elif opt == 'check_flapping':
                     watcher['check_flapping'] = dget(section, 'check_flapping',
                                                      True, bool)
                 elif opt == 'max_retry':
                     watcher['max_retry'] = dget(section, "max_retry", 5, int)
                 elif opt == 'graceful_timout':
-                    watcher['graceful_timeout'] = dget(section,
-                            "graceful_timeout", 30, int)
+                    watcher['graceful_timeout'] = dget(
+                        section, "graceful_timeout", 30, int)
                 elif opt.startswith('stderr_stream') or \
                         opt.startswith('stdout_stream'):
                     stream_name, stream_opt = opt.split(".", 1)
@@ -197,10 +224,27 @@ def get_config(config_file):
                     watcher['stream_backend'] = val
                 elif opt == 'copy_env':
                     watcher['copy_env'] = dget(section, "copy_env", False,
-                                                bool)
+                                               bool)
                 elif opt == 'copy_path':
                     watcher['copy_path'] = dget(section, "copy_path", False,
                                                 bool)
+                elif opt.startswith('hooks.'):
+                    hook_name = opt[len('hooks.'):]
+                    val = [elmt.strip() for elmt in val.split(',', 1)]
+                    if len(val) == 1:
+                        val.append(False)
+                    else:
+                        val[1] = to_boolean(val[1])
+
+                    watcher['hooks'][hook_name] = val
+
+                elif opt == 'respawn':
+                    watcher['respawn'] = dget(section, "respawn", True, bool)
+
+                elif opt == 'env':
+                    logger.warning('the env option is deprecated the use of '
+                                   'env sections is recommended')
+                    watcher['env'] = parse_env_str(val)
 
                 else:
                     # freeform
@@ -211,6 +255,20 @@ def get_config(config_file):
                 watcher['stream_backend'] = stream_backend
             watchers.append(watcher)
 
+        if section.startswith('env:'):
+            for watcher in section.split("env:", 1)[1].split(','):
+                watcher = watcher.strip()
+                if not watcher in environs:
+                    environs[watcher] = dict()
+                environs[watcher].update([(k.upper(), v)
+                                          for k, v in cfg.items(section)])
+
+    for watcher in watchers:
+        if watcher['name'] in environs:
+            if not 'env' in watcher:
+                watcher['env'] = dict()
+            watcher['env'].update(environs[watcher['name']])
+
     config['watchers'] = watchers
     config['plugins'] = plugins
     config['sockets'] = sockets
