@@ -2,6 +2,7 @@ import sys
 from Queue import Queue
 
 from circus.util import import_module, resolve_name
+from circus.stream.file_stream import FileStream
 
 
 class QueueStream(Queue):
@@ -16,19 +17,6 @@ class QueueStream(Queue):
         pass
 
 
-class FileStream(object):
-    def __init__(self, filename=None, **kwargs):
-        self._file = open(filename, 'a+')
-        self._buffer = []
-
-    def __call__(self, data):
-        self._file.write(data['data'])
-        self._file.flush()
-
-    def close(self):
-        self._file.close()
-
-
 class StdoutStream(object):
     def __init__(self, **kwargs):
         pass
@@ -65,7 +53,7 @@ def get_stream(conf):
 
 
 def get_pipe_redirector(redirect, backend='thread', extra_info=None,
-        buffer=1024):
+                        buffer=1024):
     """Redirects data received in pipes to the redirect callable.
 
     This function creates a separate thread that continuously reads
