@@ -6,10 +6,11 @@ from circus.stream.base import BaseRedirector
 
 class Redirector(BaseRedirector, Greenlet):
     def __init__(self, redirect, refresh_time=0.3, extra_info=None,
-            buffer=1024):
+                 buffer=1024):
         Greenlet.__init__(self)
         BaseRedirector.__init__(self, redirect, refresh_time=refresh_time,
-                extra_info=extra_info, buffer=buffer, selector=select)
+                                extra_info=extra_info, buffer=buffer,
+                                selector=select)
 
     def _run(self, *args, **kwargs):
         while True:
