@@ -1,7 +1,6 @@
 import subprocess
 import os
 import sys
-import time
 
 from webtest import TestApp
 
@@ -26,15 +25,20 @@ if GEVENT:
             self.app = TestApp(app)
             self.stream = QueueStream()
             # let's run a circus
-            cmd = '%s -c "from circus import circusd; circusd.main()" %s' % \
-                (sys.executable, cfg)
-            self.p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
-                                    stderr=subprocess.PIPE)
+            cmd = [sys.executable, "-c",
+                   "from circus import circusd; circusd.main()", cfg]
+            self.p = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                                      stderr=subprocess.PIPE)
 
         def tearDown(self):
             self.p.terminate()
-            self.p.kill()
-            time.sleep(0.4)
+            try:
+                with gevent.Timeout(1):
+                    while self.p.poll() is None:
+                        gevent.sleep(0.1)
+            except gevent.Timeout:
+                self.p.kill()
+
             TestCircus.tearDown(self)
 
         def test_index(self):
