@@ -6,14 +6,14 @@ import time
 import sys
 from random import randint
 
-from psutil import STATUS_ZOMBIE, STATUS_DEAD, NoSuchProcess
+from psutil import NoSuchProcess
 from zmq.utils.jsonapi import jsonmod as json
 
 from circus.process import Process, DEAD_OR_ZOMBIE, UNEXISTING
 from circus import logger
 from circus import util
 from circus.stream import get_pipe_redirector, get_stream
-from circus.util import parse_env
+from circus.util import parse_env_dict, resolve_name
 
 
 class Watcher(object):
@@ -66,6 +66,11 @@ class Watcher(object):
       - **filename**: the filename, if using a FileStream
       - **refresh_time**: the delay between two stream checks. Defaults
         to 0.3 seconds.
+      - **max_bytes**: maximum file size, after which a new output file is
+        opened. defaults to 0 which means no maximum size.
+      - **backup_count**: how many backups to retain when rotating files
+        according to the max_bytes parameter. defaults to 0 which means
+        no backups are made.
 
       This mapping will be used to create a stream callable of the specified
       class.
@@ -84,6 +89,11 @@ class Watcher(object):
       - **filename**: the filename, if using a FileStream
       - **refresh_time**: the delay between two stream checks. Defaults
         to 0.3 seconds.
+      - **max_bytes**: maximum file size, after which a new output file is
+        opened. defaults to 0 which means no maximum size.
+      - **backup_count**: how many backups to retain when rotating files
+        according to the max_bytes parameter. defaults to 0 which means
+        no backups are made.
 
       This mapping will be used to create a stream callable of the specified
       class.
@@ -125,10 +135,21 @@ class Watcher(object):
       same time.  A process will live between max_age and
       max_age + max_age_variance seconds.
 
+    - **hooks**: callback functions for hooking into the watcher startup
+      and shutdown process. **hooks** is a dict where each key is the hook
+      name and each value is a 2-tuple with the name of the callable
+      or the callabled itself and a boolean flag indicating if an
+      exception occuring in the hook should not be ignored.
+      Possible values for the hook name: *before_start*, *after_start*,
+      *before_stop*, *after_stop*.
+
     - **options** -- extra options for the worker. All options
       found in the configuration file for instance, are passed
       in this mapping -- this can be used by plugins for watcher-specific
       options.
+
+    - **respawn** -- If set to False, the processes handled by a watcher will
+      not be respawned automatically. (default: True)
     """
     def __init__(self, name, cmd, args=None, numprocesses=1, warmup_delay=0.,
                  working_dir=None, shell=False, uid=None, max_retry=5,
@@ -138,7 +159,7 @@ class Watcher(object):
                  stderr_stream=None, stream_backend='thread', priority=0,
                  singleton=False, use_sockets=False, copy_env=False,
                  copy_path=False, max_age=0, max_age_variance=30,
-                 **options):
+                 hooks=None, respawn=True, **options):
         self.name = name
         self.use_sockets = use_sockets
         self.res_name = name.lower().replace(" ", "_")
@@ -165,17 +186,21 @@ class Watcher(object):
         self.copy_path = copy_path
         self.max_age = int(max_age)
         self.max_age_variance = int(max_age_variance)
+        self.ignore_hook_failure = ['before_stop', 'after_stop']
+        self.hooks = self._resolve_hooks(hooks)
+        self.respawn = respawn
+
         if singleton and self.numprocesses not in (0, 1):
             raise ValueError("Cannot have %d processes with a singleton "
                              " watcher" % self.numprocesses)
 
         self.optnames = (("numprocesses", "warmup_delay", "working_dir",
-                      "uid", "gid", "send_hup", "shell", "env", "max_retry",
-                      "cmd", "args", "graceful_timeout", "executable",
-                      "use_sockets", "priority", "copy_env",
-                      "singleton", "stdout_stream_conf", "stderr_stream_conf",
-                      "max_age", "max_age_variance")
-                      + tuple(options.keys()))
+                          "uid", "gid", "send_hup", "shell", "env",
+                          "max_retry", "cmd", "args", "graceful_timeout",
+                          "executable", "use_sockets", "priority", "copy_env",
+                          "singleton", "stdout_stream_conf",
+                          "stderr_stream_conf", "max_age", "max_age_variance")
+                         + tuple(options.keys()))
 
         if not working_dir:
             # working dir hasn't been set
@@ -203,37 +228,62 @@ class Watcher(object):
         self.rlimits = rlimits
         self.send_hup = send_hup
         self.sockets = self.evpub_socket = None
+        self.arbiter = None
 
     def _create_redirectors(self):
         if self.stdout_stream:
             if (self.stdout_redirector is not None and
-                self.stdout_redirector.running):
+                    self.stdout_redirector.running):
                 self.stdout_redirector.kill()
-            self.stdout_redirector = get_pipe_redirector(self.stdout_stream,
-                    backend=self.stream_backend)
+            self.stdout_redirector = get_pipe_redirector(
+                self.stdout_stream, backend=self.stream_backend)
         else:
             self.stdout_redirector = None
 
         if self.stderr_stream:
             if (self.stderr_redirector is not None and
-                self.stderr_redirector.running):
+                    self.stderr_redirector.running):
                 self.stderr_redirector.kill()
 
-            self.stderr_redirector = get_pipe_redirector(self.stderr_stream,
-                    backend=self.stream_backend)
+            self.stderr_redirector = get_pipe_redirector(
+                self.stderr_stream, backend=self.stream_backend)
         else:
             self.stderr_redirector = None
 
+    def _resolve_hooks(self, hooks):
+        """Check the supplied hooks argument to make sure we can find
+        callables"""
+        if not hooks:
+            return {}
+
+        resolved_hooks = {}
+
+        for hook_name, hook_value in hooks.items():
+            callable_or_name, ignore_failure = hook_value
+
+            if callable(callable_or_name):
+                resolved_hooks[hook_name] = callable_or_name
+            else:
+                # will raise ImportError on failure
+                resolved_hook = resolve_name(callable_or_name)
+                resolved_hooks[hook_name] = resolved_hook
+
+            if ignore_failure:
+                self.ignore_hook_failure.append(hook_name)
+
+        return resolved_hooks
+
     @classmethod
     def load_from_config(cls, config):
         if 'env' in config:
-            config['env'] = parse_env(config['env'])
+            config['env'] = parse_env_dict(config['env'])
         return cls(name=config.pop('name'), cmd=config.pop('cmd'), **config)
 
     @util.debuglog
-    def initialize(self, evpub_socket, sockets):
+    def initialize(self, evpub_socket, sockets, arbiter):
         self.evpub_socket = evpub_socket
         self.sockets = sockets
+        self.arbiter = arbiter
 
     def __len__(self):
         return len(self.processes)
@@ -256,23 +306,38 @@ class Watcher(object):
             self.evpub_socket.send_multipart(multipart_msg)
 
     @util.debuglog
-    def reap_process(self, pid, status):
+    def reap_process(self, pid, status=None):
         """ensure that the process is killed (and not a zombie)"""
         process = self.processes.pop(pid)
 
+        if not status:
+            while True:
+                try:
+                    _, status = os.waitpid(pid, os.WNOHANG)
+                except OSError as e:
+                    if e.errno == errno.EAGAIN:
+                        time.sleep(0.001)
+                        continue
+                    elif e.errno == errno.ECHILD:
+                        # nothing to do here, we do not have any child
+                        # process running
+                        return
+                    else:
+                        raise
+
         # get return code
         if os.WIFSIGNALED(status):
-            retcode = os.WTERMSIG(status)
+            os.WTERMSIG(status)
         # process exited using exit(2) system call; return the
         # integer exit(2) system call has been called with
         elif os.WIFEXITED(status):
-            retcode = os.WEXITSTATUS(status)
+            os.WEXITSTATUS(status)
         else:
             # should never happen
             raise RuntimeError("Unknown process exit status")
 
         # if the process is dead or a zombie try to definitely stop it.
-        if retcode in (STATUS_ZOMBIE, STATUS_DEAD):
+        if process.status in (DEAD_OR_ZOMBIE, UNEXISTING):
             process.stop()
 
         logger.debug('reaping process %s [%s]' % (pid, self.name))
@@ -286,33 +351,13 @@ class Watcher(object):
             logger.debug('do not reap processes as the watcher is stopped')
             return
 
-        while True:
-            try:
-                # wait for completion of all the childs of circus, if it
-                # pertains to this watcher. Call reap on it.
-                pid, status = os.waitpid(-1, os.WNOHANG)
-                if not pid:
-                    return
-
-                if pid in self.processes:
-                    self.reap_process(pid, status)
-
-                if self.stopped:
-                    logger.debug('watcher have been stopped, exit the loop')
-                    return
-            except OSError as e:
-                if e.errno == errno.EAGAIN:
-                    time.sleep(0.001)
-                    continue
-                elif e.errno == errno.ECHILD:
-                    return
-                else:
-                    raise
+        # reap_process changes our dict, look through the copy of keys
+        for pid in self.processes.keys():
+            self.reap_process(pid)
 
     @util.debuglog
     def manage_processes(self):
-        """ manage processes
-        """
+        """Manage processes."""
         if self.stopped:
             return
 
@@ -326,14 +371,14 @@ class Watcher(object):
                                        "time": time.time()})
                     self.kill_process(process)
 
-        if len(self.processes) < self.numprocesses:
+        if self.respawn and len(self.processes) < self.numprocesses:
             self.spawn_processes()
 
         processes = self.processes.values()
         processes.sort()
         while len(processes) > self.numprocesses:
             process = processes.pop(0)
-            if process.status == STATUS_DEAD:
+            if process.status == DEAD_OR_ZOMBIE:
                 self.processes.pop(process.pid)
             else:
                 self.processes.pop(process.pid)
@@ -341,8 +386,7 @@ class Watcher(object):
 
     @util.debuglog
     def reap_and_manage_processes(self):
-        """Reap & manage processes.
-        """
+        """Reap & manage processes."""
         if self.stopped:
             return
         self.reap_processes()
@@ -378,11 +422,11 @@ class Watcher(object):
             process = None
             try:
                 process = Process(self._process_counter, cmd,
-                          args=self.args, working_dir=self.working_dir,
-                          shell=self.shell, uid=self.uid, gid=self.gid,
-                          env=self.env, rlimits=self.rlimits,
-                          executable=self.executable, use_fds=self.use_sockets,
-                          watcher=self)
+                                  args=self.args, working_dir=self.working_dir,
+                                  shell=self.shell, uid=self.uid, gid=self.gid,
+                                  env=self.env, rlimits=self.rlimits,
+                                  executable=self.executable,
+                                  use_fds=self.use_sockets, watcher=self)
 
                 # stream stderr/stdout if configured
                 if self.stdout_redirector is not None:
@@ -397,7 +441,7 @@ class Watcher(object):
 
                 self.processes[process.pid] = process
                 logger.debug('running %s process [pid %d]', self.name,
-                            process.pid)
+                             process.pid)
             except OSError, e:
                 logger.warning('error in %r: %s', self.name, str(e))
 
@@ -433,7 +477,7 @@ class Watcher(object):
             # now sending the signal to the process itself
             self.send_signal(process.pid, sig)
             self.notify_event("kill", {"process_pid": process.pid,
-                                   "time": time.time()})
+                                       "time": time.time()})
 
         except NoSuchProcess:
             # already dead !
@@ -500,7 +544,7 @@ class Watcher(object):
 
     @util.debuglog
     def info(self):
-        return dict([(proc.pid, proc.info())\
+        return dict([(proc.pid, proc.info())
                      for proc in self.processes.values()])
 
     @util.debuglog
@@ -521,6 +565,9 @@ class Watcher(object):
         logger.debug('gracefully stopping processes [%s] for %ss' % (
                      self.name, self.graceful_timeout))
 
+        # We ignore the hook result
+        self.call_hook('before_stop')
+
         while self.get_active_processes() and time.time() < limit:
             self.kill_processes(signal.SIGTERM)
             try:
@@ -536,6 +583,8 @@ class Watcher(object):
 
         self.stopped = True
 
+        # We ignore the hook result
+        self.call_hook('after_stop')
         logger.info('%s stopped', self.name)
 
     def get_active_processes(self):
@@ -548,6 +597,28 @@ class Watcher(object):
         """Returns a list of PIDs"""
         return [process.pid for process in self.processes]
 
+    def call_hook(self, hook_name):
+        """Call a hook function"""
+        kwargs = {'watcher': self, 'arbiter': self.arbiter,
+                  'hook_name': hook_name}
+
+        if hook_name in self.hooks:
+            try:
+                result = self.hooks[hook_name](**kwargs)
+                error = None
+                self.notify_event("hook_success",
+                                  {"name": hook_name, "time": time.time()})
+            except Exception, error:
+                logger.exception('Hook %r failed' % hook_name)
+                result = hook_name in self.ignore_hook_failure
+                self.notify_event("hook_failure",
+                                  {"name": hook_name, "time": time.time(),
+                                   "error": str(error)})
+
+            return result
+        else:
+            return True
+
     @util.debuglog
     def start(self):
         """Start.
@@ -556,9 +627,20 @@ class Watcher(object):
             return
 
         self.stopped = False
+
+        if not self.call_hook('before_start'):
+            logger.debug('Aborting startup')
+            self.stopped = True
+            return False
+
         self._create_redirectors()
         self.reap_processes()
-        self.manage_processes()
+        self.spawn_processes()
+
+        if not self.call_hook('after_start'):
+            logger.debug('Aborting startup')
+            self.stop()
+            return False
 
         if self.stdout_redirector is not None:
             self.stdout_redirector.start()
@@ -568,6 +650,7 @@ class Watcher(object):
 
         logger.info('%s started' % self.name)
         self.notify_event("start", {"time": time.time()})
+        return True
 
     @util.debuglog
     def restart(self):
@@ -575,8 +658,10 @@ class Watcher(object):
         """
         self.notify_event("restart", {"time": time.time()})
         self.stop()
-        self.start()
-        logger.info('%s restarted', self.name)
+        if self.start():
+            logger.info('%s restarted', self.name)
+        else:
+            logger.info('Failed to restart %s', self.name)
 
     @util.debuglog
     def reload(self, graceful=True):
